/*
    $Id: progressbar.c,v 1.3 2003/01/12 14:54:58 cramer Exp $

    Displays a progressbar

    Copyright (C) 2002 Matthias Cramer, cramer@freestone.net
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
                     
int get_columns();

/*
Draws a progressbar with has a width of length. if length is 0 then
the with is as with as the terminal. max is the maximum value with represents 100%
and curr is the current value.
*/

char * progressbar(int length, int max, int curr) {
  static char buf[255];
  char bar[255];
  int i, proc, barlength;
  
  buf[0]=0;
  bar[0]=0;
  
  if (length==0) {
    length=get_columns();
  }
  
  if (curr == 0) {
    proc = 0;
    barlength = 0;
  } else {
    proc = (int) (curr * 100.0 / max);
    barlength = (int) ((curr * 1.0/ max * 1.0) * (length-7.0));
  }

  for(i=1; i<=(length-7); i++) {
    if ( i<=barlength ) {
      strncat(bar, "#", 1);
    } else {
      strncat(bar, "-", 1);
    }
  }

  snprintf(buf, 255, "|%s| %03d%%", bar, proc);
  return(buf);

}


/*
Determines the number of columns of /dev/tty.
Returns the number of columns, or -1 if error.
*/

int get_columns() {

  struct winsize win;
  int fd, result;

#ifdef TIOCGWINSZ
  fd = open("/dev/tty",O_WRONLY);
  if (fd < 0) return -1;
  result = ioctl(fd,TIOCGWINSZ,&win);
  close(fd);
  if (result != 0) return -1;
#else
  win.ws_col = 80;
#endif
  return win.ws_col;
}
